module TFT_CTRL(  //采用其他输出均滞后Data_Req的方式进行时序对齐，梅哥推荐写法
	PCLK, //TFT像素时钟9M 4.3寸屏 分辨率480*272
	Rst_n,
	Data_in, //输入为16位RGB
	
	TFT_HS,	//行同步信号
	TFT_VS,	//场同步信号
	TFT_BL,	//背光 （可以是1K PWM，现在直接接高就行）
	TFT_RGB,	//送给TFT显示的数据RGB565
	TFT_PCLK,	//显示屏像素驱动时钟 33.3MHZ
	Data_En,		//传输显示区域有效数据(480*272)
	X_Count,		//显示区域有效数据的行坐标X  同步与Data_En
	Y_Count,		//显示区域有效数据的列坐标Y
	Data_Req   //所有其他输出均滞后Data_Req两拍，而且Data_Req是准时的。
);
	//注意驱动TFT的时钟是9M，不是50M。顶层要加一个PLL  程序采用组合逻辑实现

	input PCLK;
	input Rst_n;
	input [15:0]Data_in; 
	
	output TFT_HS;	//行同步信号
	output TFT_VS;	//场同步信号
	output TFT_BL;	//背光 （可以是1K PWM，现在直接接高就行）
	output [15:0]TFT_RGB;	//送给TFT显示的数据，实际用的是RGB565 只有16位
	output TFT_PCLK;	//显示屏像素驱动时钟 9MHZ
	output Data_En;		//当扫到显示区480*272坐标时，通知TFT显示屏芯片此时送的数据都是有效显示数据
	output reg[8:0]X_Count;		//显示区域有效数据的行坐标X 480
	output reg[8:0]Y_Count;		//显示区域有效数据的列坐标Y 272
	output Data_Req;//最大可以延迟三拍
	
	localparam HS_pulse = 41;
	localparam HS_back_porch = 2;
	localparam HS_left_border = 0;
	localparam HS_right_border = 0;
	localparam HS_front_porch = 2;
	localparam HS_data_time = 480;//总共是1056
	localparam HS_total_time = 525;
	

	
	localparam VS_pulse = 10;
	localparam VS_back_porch = 2;
	localparam VS_top_border = 0;
	localparam VS_bottom_border = 0;
	localparam VS_front_porch = 2;
	localparam VS_data_time = 272; //总共是525
	localparam VS_total_time = 286;
	
		parameter 
		HS_end = HS_pulse - 1,
		Hdat_begin = HS_pulse + HS_back_porch + HS_left_border - 1,
		Hdat_end = HS_pulse + HS_back_porch + HS_left_border + HS_data_time- 1,
		
		VS_end = VS_pulse - 1,
		Vdat_begin = VS_pulse + VS_back_porch + VS_top_border - 1,
		Vdat_end = VS_pulse + VS_back_porch + VS_top_border + VS_data_time - 1;
	

	
	reg [9:0]HS_cnt; //最大到525-1
	reg [8:0]VS_cnt;	//最大到286-1
	reg [3:0]Data_En_r;
	wire TFT_HS_tmp,TFT_VS_tmp;
	reg [3:0]TFT_HS_r;
	reg [3:0]TFT_VS_r;
	
	
	always@(posedge PCLK or negedge Rst_n)
	if(!Rst_n)
		HS_cnt <= 10'd0;
	else if(HS_cnt >= HS_total_time-1)
		HS_cnt <= 10'd0;
	else 
		HS_cnt <= HS_cnt + 1'b1;
		
	always@(posedge PCLK or negedge Rst_n)
	if(!Rst_n)
		VS_cnt <= 9'd0;	
	else if(HS_cnt >= HS_total_time-1)begin//行记满则列加1
		if(VS_cnt >= VS_total_time-1)
			VS_cnt <= 9'd0;
		else
			VS_cnt <= VS_cnt + 1'b1;
	end
		


	assign Data_Req = ( HS_cnt > Hdat_begin  && HS_cnt <= Hdat_end && VS_cnt > Vdat_begin && VS_cnt <= Vdat_end);//目前Data_Req是严格符合VGA时序准时的，用做RAM的rden
	
	always@(posedge PCLK or negedge Rst_n)
	if(!Rst_n)
		Data_En_r[3:0] <= 4'd0;
	else begin
		Data_En_r[0] <= Data_Req;
		Data_En_r[3:1] <= Data_En_r[2:0];
	end
	
	assign Data_En = Data_En_r[1];//Data_Req延迟两拍得到Data_En
	
	
	
//	//	X_Count_r是准时的，但X_Count就滞后一拍
//	assign X_Count_r = Data_En?HS_cnt - Hdat_begin - 1:0; //保证坐标从0-479开始 为了串口传图实验，修正此处bug assign X_Count = Data_En?HS_cnt - Hdat_begin - 1:0;
//	assign Y_Count_r = Data_En?VS_cnt - Vdat_begin - 1:Y_Count;//保证坐标从0-271开始		在Data_En无效时刻其实没必要清零X,Y坐标计数值，否则后级模块看到清零值会认为此刻扫描坐标为0行零列，会出现误判。因此改用时序逻辑
	
	always@(posedge PCLK or negedge Rst_n)
	if(!Rst_n)
		X_Count <= 9'd0;		
	else 
		X_Count <= Data_En_r[0]?HS_cnt - Hdat_begin - 1:0;//因为X为0的前两拍要用来提前产生rden信号，所以X_Count到499后在非使能时刻必须为0
	//X_Count必须归0，那就主要借助Y_Count规避Y清零时对RAM的额外读写
	//因为Y_Count <= Data_En?VS_cnt - Vdat_begin - 1:Y_Count;时Y_Count滞后Data_En一拍，为了对齐，所以参考Data_En的前一拍
	
	
	always@(posedge PCLK or negedge Rst_n)//仿真发现当Y为271记满后VS拉低并没有及时归0，导致RAM读取出现漏读。因为X的Data_En本身就与TFT_HS_tmp相同，所以会自动归0
	if(!Rst_n)
		Y_Count <= 9'd0;
	else if(TFT_VS_tmp)
		Y_Count <= Data_En_r[0]?VS_cnt - Vdat_begin - 1:Y_Count;
	else 
		Y_Count <= 9'd0;

	
	assign TFT_HS_tmp = HS_cnt <= HS_end?0:1;
	assign TFT_VS_tmp = VS_cnt <= VS_end?0:1;
	assign TFT_RGB = Data_En?Data_in:16'd0;
	
	always@(posedge PCLK or negedge Rst_n)
	if(!Rst_n)begin
		TFT_HS_r <= 4'd0;
		TFT_VS_r <= 4'd0;
	end 
	else begin 
		TFT_HS_r[0] <= TFT_HS_tmp;
		TFT_VS_r[0] <= TFT_VS_tmp;
		TFT_HS_r[3:1] <= TFT_HS_r[2:0];
		TFT_VS_r[3:1] <= TFT_VS_r[2:0];
	end 
	
	assign TFT_HS = TFT_HS_r[1];
	assign TFT_VS = TFT_VS_r[1];
		
	
	
	
	assign TFT_PCLK = PCLK;
	assign TFT_BL = 1;
	


endmodule 
