module TFT_CTRL_Top( //模块中产生色块作为激励源，所以无需外部输入RGB数据
	Clk,
	Rst_n,
	
	TFT_HS,	//行同步信号
	TFT_VS,	//场同步信号
	TFT_BL,	//背光 （可以是1K PWM，现在直接接高就行）
	TFT_RGB,	//送给TFT显示的数据RGB565
	TFT_PCLK,	//显示屏像素驱动时钟 33.3MHZ
	Data_En	
);

	input Clk;
	input Rst_n;
	
	output TFT_HS;	//行同步信号
	output TFT_VS;	//场同步信号
	output TFT_BL;	//背光 （可以是1K PWM，现在直接接高就行）
	output [15:0]TFT_RGB;	//送给TFT显示的数据RGB565
	output TFT_PCLK;	//显示屏像素驱动时钟 33.3MHZ
	output Data_En;
	
	wire [8:0]X_Count;		//显示区域有效数据的行坐标X 800
	wire [8:0]Y_Count;
	reg [15:0]Data_in;
	wire PCLK;
	wire PLL_lock;
	
	localparam
		BLACK = 16'h0000, //黑色
		BLUE = 16'h001F, //蓝色
		RED = 16'hF800, //红色
		PURPPLE = 16'hF81F, //紫色
		GREEN = 16'h07E0, //绿色
		CYAN = 16'h07FF, //青色
		YELLOW = 16'hFFE0, //黄色
		WHITE = 16'hFFFF; //白色
		
	tft_pll	tft_pll_inst (//PLL是高电平复位，所以要取反
	.areset ( ~Rst_n ),
	.inclk0 ( Clk ),
	.c0 ( PCLK ),
	.locked ( PLL_lock )
	);
		
		
	TFT_CTRL TFT_CTRL_inst( //注意驱动TFT的时钟是33.3M，不是50M。顶层要加一个PLL
		.PCLK(PCLK), //TFT像素时钟33.3M 五寸屏
		.Rst_n(PLL_lock),
		.Data_in(Data_in), 
		
		.TFT_HS(TFT_HS),	//行同步信号
		.TFT_VS(TFT_VS),	//场同步信号
		.TFT_BL(TFT_BL),	//背光 （可以是1K PWM，现在直接接高就行）
		.TFT_RGB(TFT_RGB),	//送给TFT显示的数据RGB565
		.TFT_PCLK(TFT_PCLK),	//显示屏像素驱动时钟 33.3MHZ
		.Data_En(Data_En),		//传输显示区域有效数据(800*480)
		.X_Count(X_Count),		//显示区域有效数据的行坐标X
		.Y_Count(Y_Count),		//显示区域有效数据的列坐标Y
		.Data_Req()
	);	
	//R表示行，C表示列 区域分为2*4，显示8种颜色色块 272/4 = 68 480/2 = 240
	wire R0 = (Y_Count >= 0 ) & (Y_Count < 68 );//从0开始左大于等于，右小于。 从左边界开始左大于，右小于等于
	wire R1 = (Y_Count >= 68 ) & (Y_Count < 68*2 );
	wire R2 = (Y_Count >= 68*2 ) & (Y_Count < 68*3 );
	wire R3 = (Y_Count >= 68*3 ) & (Y_Count < 68*4 );
	
	wire C0 = (X_Count >= 0) & (X_Count < 240 );
	wire C1 = (X_Count >= 240 ) & (X_Count < 240*2 );
	
	//这8个信号同一时间只有一个可以是高电平激活
	wire R0_C0_act = R0 & C0;
	wire R0_C1_act = R0 & C1;
	wire R1_C0_act = R1 & C0;
	wire R1_C1_act = R1 & C1;
	wire R2_C0_act = R2 & C0;
	wire R2_C1_act = R2 & C1;
	wire R3_C0_act = R3 & C0;
	wire R3_C1_act = R3 & C1;
	
	always@(*)
		case({R3_C1_act,R3_C0_act,R2_C1_act,R2_C0_act,R1_C1_act,R1_C0_act,R0_C1_act,R0_C0_act})
			8'b0000_0001:Data_in = BLACK;
			8'b0000_0010:Data_in = BLUE;
			8'b0000_0100:Data_in = RED;
			8'b0000_1000:Data_in = PURPPLE;
			8'b0001_0000:Data_in = GREEN;
			8'b0010_0000:Data_in = CYAN;
			8'b0100_0000:Data_in = YELLOW;
			8'b1000_0000:Data_in = WHITE;
			default:Data_in = BLACK;
		endcase 
	
	
	





endmodule
