`timescale 1ns/1ns
module UART_RAM_TFT_Top( //因为串口软件传数据时标点符合，以及16进制的0x前缀都是无法传输。串口发32768个数 实验发现串口接收模块可能有问题
	Clk,
	Rst_n,
	Rs232_Rx,
	
	TFT_HS,	//行同步信号
	TFT_VS,	//场同步信号
	TFT_BL,	//背光 （可以是1K PWM，现在直接接高就行）
	TFT_RGB,	//送给TFT显示的数据RGB565
	TFT_PCLK,	//显示屏像素驱动时钟 33.3MHZ
	Data_En
);
	parameter Req_Early_En = 2;
	parameter BMP_Xmax = 127;//X结束坐标，从0开始 256*256存不下，所以用128*128的16位数据=16384 所以串口要发32768次
	parameter BMP_Ymax = 127;	//Y结束坐标，从0开始
	
	input Clk;
	input Rst_n;
	input Rs232_Rx;
	
	output TFT_HS;	//行同步信号
	output TFT_VS;	//场同步信号
	output TFT_BL;	//背光 （可以是1K PWM，现在直接接高就行）
	output [15:0]TFT_RGB;	//送给TFT显示的数据RGB565
	output TFT_PCLK;	//显示屏像素驱动时钟 33.3MHZ
	output Data_En;
	
	wire Rx_done;
	wire [7:0]Rx_data;
	
	wire wr_en;
	wire [15:0]wr_addr;//写入RAM的16位数据16384个，65536个存不下
	wire wr_full;
	wire [15:0]wr_data;
	 
	
	reg [15:0]rdaddress;
	wire rden;
	wire [15:0]q;
	
	wire pll_lock;
	wire PCLK;
	
	wire Data_Req;
	wire [8:0]X_Count;		//显示区域有效数据的行坐标X 800
	wire [8:0]Y_Count;		//显示区域有效数据的列坐标Y 480
	wire [15:0]Data_in;
	
	uart_byte_rx uart_byte_rx0(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.Rs232_Rx(Rs232_Rx),//串口接收数据脚
		.Baud_set(3'd4),//波特率值设置115200
		//此处使能接收信号由下降沿作为Receive_en标志
		.Data_byte(Rx_data),//串口接收到的数据
		.Rx_Done(Rx_done)//串口接收完成标志信号
	);
	
	uart2ram uart2ram_inst(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.Rx_done(Rx_done),
		.Rx_data(Rx_data),
		
		.wr_en(wr_en),
		.wr_addr(wr_addr),
		.wr_data(wr_data),
		.wr_full()
	);
	//其实RAM的读使能和写使能都拉高也没问题，只要将非正常情况RAM送出的数据不送入TFT端口显示即可，但是我为了保险起见还是加以控制
	dpram	dpram_inst (//因此当读使能无效后由于RAM读出数据变化滞后读地址一拍，所以在读使能无效后仍然会接着输出一个数据，保证数据个数完整性。而且读使能无效后输出q一直保持最后的65535值不变。
	.data ( wr_data ),
	.wraddress ( wr_addr ),
	.wrclock ( Clk ),
	.wren ( wr_en ),
	
	.rd_aclr ( 1'b0 ),
	.rdaddress ( rdaddress ),
	.rdclock ( PCLK ),//读时钟用PLL产生的***
	.rden ( rden ),
	.q ( q )//输出给TFT的
	
	
	);

	tft_pll	tft_pll_inst (
	.areset ( ~Rst_n ),
	.inclk0 ( Clk ),
	.c0 ( PCLK ),//作为TFT的时钟***
	.locked ( pll_lock )//作为TFT的复位***
	);
	
	TFT_CTRL TFT_CTRL_inst( //注意驱动TFT的时钟是33.3M，不是50M。顶层要加一个PLL
		.PCLK(PCLK), //TFT像素时钟9M 五寸屏***
		.Rst_n(pll_lock),//****
		.Data_in(Data_in), 
		
		.TFT_HS(TFT_HS),	//行同步信号
		.TFT_VS(TFT_VS),	//场同步信号
		.TFT_BL(TFT_BL),	//背光 （可以是1K PWM，现在直接接高就行）
		.TFT_RGB(TFT_RGB),	//送给TFT显示的数据RGB565
		.TFT_PCLK(TFT_PCLK),	//显示屏像素驱动时钟 33.3MHZ
		.Data_En(Data_En),		//传输显示区域有效数据(800*480)
		
		.X_Count(X_Count),		//显示区域有效数据的行坐标X
		.Y_Count(Y_Count),		//显示区域有效数据的列坐标Y
		.Data_Req(Data_Req)
	);
	
	//仿真中发现当Y_count记满255后再记到256时理论上address不再增加，应该维持在65535。但是由于每行都有Data_Req超前Data_En两拍，导致255列后面的每行address都加2，解决办法是让X_count和Y_count在Data_En无效时仍然保持原值，这就要把TFT_CTRL的组合逻辑改成时序逻辑了
	always@(posedge PCLK or negedge Rst_n)//写法1//////////读时钟也是PLL产生*************************************tb仿真时提升速度记得屏蔽PLL例化代码，把这里的敏感时钟也改为Clk，漏掉仿真时RAM读取就出问题了
	if(!Rst_n)//仿真后下板记得再把时钟还原为PLL的PCLK 9M，否则板子白色无现象.
		rdaddress <= 16'd0;//现在每一行读完虽然rdaddress值为256，但是这个256对应的值没有被用来显示，从而确保下一行开始没有丢数据，正好是从地址256开始
	else if(Data_Req && X_Count<=BMP_Xmax-Req_Early_En && Y_Count <= BMP_Ymax )
		if(~Data_En && Y_Count == BMP_Ymax)//剔除最后的两个害群之马
			rdaddress <= 16'd0;
		else 
			rdaddress <= rdaddress + 1'd1;//rdaddress又滞后X_Count(255-Req_Early_En=253)一拍，Data_Req又提前2拍，所以rdaddress加了2+(0-253)254=256次，最后结果就是rdaddress为256

	/////////////////////////////////////////////////////////////////////////仿真最后发现超前发出Data_Req缺点很多，时序凌乱，Bug非常多，回天乏术.因此改用小梅哥的滞后思想	



		
	assign rden =  Data_Req;
	
	//发现rden拉高时间过长，导致RAM地址为0的数据不断在X坐标256以后的点上显示，这就是rden控制不当造成。
	assign Data_in = (X_Count<=BMP_Xmax && Y_Count <= BMP_Ymax)?q:16'd0;
	
endmodule 