module uart2ram(
	Clk,
	Rst_n,
	Rx_done,
	Rx_data,
	
	wr_en,
	wr_addr,
	wr_data,
	wr_full //这个信号TFT扫描用不到，但是其他场合会用到。TFT每秒都会从RAM中读数据，不需要等到一帧图片全部存入RAM后才能读出
);

	input Clk;
	input Rst_n;
	input Rx_done;
	input [7:0]Rx_data;
	
	output reg wr_en;
	output reg[15:0]wr_addr;//写入RAM的16位数据共65536个
	output reg wr_full;
	output [15:0]wr_data;
	
	reg [16:0]rx_cnt;//串口接收计数，因为一次收8位，需要计数到256*256*2 = 2^17-1,需17位.
	reg [15:0]data_tmp;
	
	parameter PIX_CNT_MAX = 18'd32768;//按照8位进行统计

	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		rx_cnt <= 17'd0;
	else if(Rx_done)begin
		if(rx_cnt >= PIX_CNT_MAX-1)
			rx_cnt <= 17'd0;
		else
			rx_cnt <= rx_cnt + 1'd1;
	end
	
	
	//时序逻辑滞后一拍，导致data_tmp出现预期外的中间组合值，所以要控制wr_en进行精准控制
	always@(posedge Clk or negedge Rst_n)//拼成16位 注意串口先发的是图像RGB数据的高8位，后发低八位。
	if(!Rst_n)
		data_tmp <= 16'd0;
	else if(Rx_done)begin
		data_tmp[7:0] <= Rx_data;
		data_tmp[15:8] <= data_tmp[7:0];
		//		data_tmp <= {data_tmp[7:0],Rx_data};
	end
	
	assign wr_data = data_tmp;
	//assign data_tmp = {data_tmp[7:0],Rx_data};//data_tmp值是连续俩个相同的Rx_data
	
	//wr_full也是单脉冲
	always@(posedge Clk or negedge Rst_n)//当串口接收到的数据都存入RAM后产生wr_full告知外界可以来RAM读取数据了。
	if(!Rst_n)
		wr_full <= 1'd0;
	else if(rx_cnt >= PIX_CNT_MAX-1 && Rx_done)
		wr_full <= 1'd1;
	else 
		wr_full <= 1'd0;
		//读地址这样用组合逻辑凑不好理解，而且当需求改为每三个数据组合起来存入一次RAM组合逻辑就不行了。建议时序逻辑更简单,有效地址也只需要产生参考一个精确的单脉冲即可。
	//assign wr_addr = rx_cnt[16:1];//组合逻辑存的第一个数地址是1开始不是从0开始
	//用时序逻辑的话写使能和写地址都可以同步为单脉冲，逻辑清楚。
	always@(posedge Clk or negedge Rst_n)//当串口接收到的数据都存入RAM后产生wr_full告知外界可以来RAM读取数据了。
	if(!Rst_n)
		wr_addr <= 16'd0;	
	else if(Rx_done&&rx_cnt[0])//只需要一个时钟周期的高脉冲写使能就可以把数据写进去了
		wr_addr <= rx_cnt[16:1];	
	///////当需求改为每三个数据组合起来存入一次RAM时采用模三计数器，cnt记到2就让addr加1即可。
	
	
	
	
	
	
	
	//写使能时间长度有问题
	//assign wr_en = (rx_cnt > 1'b0或者1都不行) & (rx_cnt <= PIX_CNT_MAX-1);//因为wr_en有效时刻是cnt计数从1到131072。
	//assign wr_en = rx_cnt[1];//会在一开头rx_cnt为初始值0时写入一个无效数据
	
	//前面考虑的方向错了，wr_en不需要在wr_data有效时刻一直维持高电平，只需要是一个单周期脉冲即可
	always@(posedge Clk or negedge Rst_n)//当串口接收到的数据都存入RAM后产生wr_full告知外界可以来RAM读取数据了。
	if(!Rst_n)
		wr_en <= 1'd0;	
	else if(Rx_done&&rx_cnt[0])//只需要一个时钟周期的高脉冲写使能就可以把数据写进去了
		wr_en <= 1'd1;
	else 
		wr_en <= 1'd0;
		
	//	assign wr_en = (rx_cnt <= 18'd131072);//如果用这个每次传输完后的数据会被再次写入地址0的RAM里，不符合预期。
	

endmodule 
