module uart_byte_rx(//优化功能：完全保证计数160个脉冲，bps_cnt跳变与bps_clk上升沿一致，从而串口接收采样处与case分支相对应,但是Rx_Done仍超前Tx_Done,这是由于波特率时钟计算时舍弃小数位导致的，无法避免
	Clk,
	Rst_n,
	Rs232_Rx,//串口接收数据脚
	Baud_set,//波特率值设置
	//此处使能接收信号由下降沿作为Receive_en标志
	Data_byte,//串口接收到的数据
	Rx_Done//串口接收完成标志信号
);

	input Clk;
	input Rst_n;
	input Rs232_Rx;
	input [2:0]Baud_set;
	
	output reg[7:0]Data_byte;
	output reg Rx_Done;
	
	reg [8:0]bps_DR;
	reg s0_Rs232_Rx,s1_Rs232_Rx;
	reg tmp0_Rs232_Rx,tmp1_Rs232_Rx;
	reg UART_state;
	reg [8:0]Div_cnt;
	reg bps_clk;
	reg [7:0]bps_cnt;//对16倍波特率时钟计数，最大为0-161，0是默认初值，真正生效采样时刻的是1-160，有用的共160个时间间隔，即16*10=160，最后的161是为了保证160的维持时间，从而确保Rx_Done滞后Tx_Done
	reg [2:0]Data_byte_r[7:0];//定义一个二维数组，前面的[2:0]指定数组中每一个元素的位宽是3位，即每个元素的数据最大是7，后面的[7:0]指定数组中共有8个元素。	
//二维数组的列，即元素的大小//二维数组的行,即元素个数
	reg [2:0]START_BIT;//串口接收起始位
	reg [2:0]STOP_BIT;//串口接收停止位
	
	wire Receive_En;
	
	//产生16倍过采样计数值
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		bps_DR <= 9'd324;//按照9600的波特率
	else begin
		case(Baud_set)
			0:bps_DR <= 9'd324;//按照9600的波特率
			1:bps_DR <= 9'd162;//按照9600的波特率
			2:bps_DR <= 9'd80;//按照9600的波特率
			3:bps_DR <= 9'd53;//按照9600的波特率
			4:bps_DR <= 9'd26;//按照115200的波特率
			default:bps_DR <= 9'd324;//默认按照9600的波特率
		endcase 

	end 
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)begin
		s0_Rs232_Rx <= 1'b0;
		s1_Rs232_Rx <= 1'b0;
	end
	else begin
		s0_Rs232_Rx <= Rs232_Rx;
		s1_Rs232_Rx <= s0_Rs232_Rx;
	end	
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)begin
		tmp0_Rs232_Rx <= 1'b0;
		tmp1_Rs232_Rx <= 1'b0;
	end
	else begin
		tmp0_Rs232_Rx <= s1_Rs232_Rx;
		tmp1_Rs232_Rx <= tmp0_Rs232_Rx;
	end	

	assign Receive_En = !tmp0_Rs232_Rx & tmp1_Rs232_Rx;

	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		UART_state <= 1'b0;
	else if(Receive_En)
		UART_state <= 1'b1;
	else if(bps_cnt == 8'd161 || (bps_cnt == 8'd12 && START_BIT > 3'd2))//这样UART_state才能与Rx_Done信号的跳变沿对齐，否则若为else if(Rx_Done || (bps_cnt == 8'd12 && (START_BIT > 2)))则由于Rx_Done对于bps_cnt == 159捕获的延迟，UART_state与Rx_Done的边沿就错位不对齐
//	else if(bps_cnt == 8'd159 || START_BIT > 3'd2)//除啦bps_cnt记满159完成一次完整的串口接收外，如果起始位接收异常，说明并非真正的串口接收信号，也要清零UART_state，让串口回到接收空闲态
		UART_state <= 1'b0;//当起始位有3个1及以上，表明起始位大概率不是0，说明接收有问题
	else
		UART_state <= UART_state;
		
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		Div_cnt <= 9'd0;
	else if(UART_state)begin//如果串口开始接收，就要启动串口波特率时钟分频计数模块
		if(Div_cnt == bps_DR)
			Div_cnt <= 9'd0;
		else
			Div_cnt <= Div_cnt + 1'd1;
	end
	else
		Div_cnt <= 9'd0;
	
	always@(posedge Clk or negedge Rst_n)//由于波特率时钟计算时并非精准的16倍，所以舍弃了小数位，因此串口接收必须要用多次过采样取中间平均值的方法减少误差
	if(!Rst_n)
		bps_clk <= 1'd0;
	else if(Div_cnt == 9'd1)//当启动串口接收后立刻拉高	bps_clk产生高电平脉冲
		bps_clk <= 1'd1;
	else
		bps_clk <= 1'd0;

	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		bps_cnt <= 8'd0;
	else if(bps_cnt == 8'd161 || (bps_cnt == 8'd12 && START_BIT > 3'd2))//这种写法更精确
//	else if(bps_cnt == 8'd159 || START_BIT > 3'd2)//如果bps_cnt记满159或者串口起始位接收异常，也要清零串口接收进程寄存器bps_cnt//此处不可用Rx_Done信号作为bps_cnt的清零信号，否则会多一个D触发器，导致输出多延迟一个时钟周期
		bps_cnt <= 8'd0;
//	else if(bps_clk)//当启动串口接收后bps_cnt值立刻由0变为1，然后增加,但是bps_cnt的变化沿会滞后于bps_clk的高电平，导致采样错位
	else if(Div_cnt == 9'd1)//采用这种方案，bps_cnt变化就与bps_clk的上升沿同步，case的分支就按照现在的6,7,8,9,10,11即可，否则改用上面小梅哥的方案由于bps_cnt就会滞后bps_clk,因此当前bps_clk为高电平进入case分支采到串口接收端的电平就会是前一个bps_cnt快结束的末尾时刻的电平值，
	//不应该算到下一个bps_cnt的头上，所以case分支就要统一提前为5,6,7,8,9,10。这是因为此时起始位被采样的bps_cnt范围是0-15，中间6位对应5,6,7,8,9,10。按照小梅哥提供串口接收文档的起始位被采样的bps_cnt范围是1-16，中间六位是6,7,8,9,10,11，因为bps_cnt默认初值就是0，当使能串口接收后
	//bps_cnt立刻变为1开始采样串口接收端电平，而实际仿真波形发现，bps_clk的高电平超前bps_cnt的变化，因此第一次采样时bps_cnt的值仍是0的最后阶段，并非1，所以所有case分支都要减1才能真正对应上，这样修改的地方太多，可以修改代码使bps_cnt的变化沿与bps_clk的上升沿对齐，数据采样的位置就不发生错位了
		bps_cnt <= bps_cnt + 8'd1;
	else
		bps_cnt <= bps_cnt;
		
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		Rx_Done <= 1'd0;//bps_cnt == 8'd162可保证Rx_Done滞后于Tx_Done高电平脉冲，符合理论
	else if(bps_cnt == 8'd161)//最好保证Rx_Done滞后于Tx_Done高电平脉冲，符合理论，实际上接收完成标志可以提前于发送完成标志，但为啦满足将接收完成信号接到发送模块的发送使能端口实现高效率的连续发送接收功能这一应用场景，必须保证发送完成信号在接收完成信号之前，否则还没有发送完整的一帧数据就又启动下一次发送会导致发送数据塞车，出现发送接收异常的问题
		Rx_Done <= 1'd1;
	else
		Rx_Done <= 1'd0;
		
	always@(posedge Clk or negedge Rst_n)//不要忘了起始位和停止位
	if(!Rst_n)begin//给二维数组的8个元素赋初值
		START_BIT <= 3'd0;
		Data_byte_r[0] <= 3'd0;
		Data_byte_r[1] <= 3'd0;
		Data_byte_r[2] <= 3'd0;
		Data_byte_r[3] <= 3'd0;
		Data_byte_r[4] <= 3'd0;
		Data_byte_r[5] <= 3'd0;
		Data_byte_r[6] <= 3'd0;
		Data_byte_r[7] <= 3'd0;
		STOP_BIT <= 3'd0;
	end
	else if(bps_clk)
//	else if(UART_state)//此处由于程序上每20ns就触发一次，因为串口接收时UART_state一直为1，因此case的分支就会重复累加执行多次，应修改为每当bps_clk为高电平时可以进入case分支才能保证一共进入160次case，符合理论设计
		case(bps_cnt)
			1://串口接收一次就累加一次，如果不清零就会加满溢出，所以在第一次启动串口接收时进行清零操作，保证接收的数据是干净的，不会有之前接收数据时的残留，因为小梅哥是从0-159有用，而我采用1-160有用，因此我要在1时清零，当然在最后161处清零也可以，但默认都是最开始清零
				begin
					START_BIT <= 3'd0;
					Data_byte_r[0] <= 3'd0;
					Data_byte_r[1] <= 3'd0;
					Data_byte_r[2] <= 3'd0;
					Data_byte_r[3] <= 3'd0;
					Data_byte_r[4] <= 3'd0;
					Data_byte_r[5] <= 3'd0;
					Data_byte_r[6] <= 3'd0;
					Data_byte_r[7] <= 3'd0;
					STOP_BIT <= 3'd0;
				end
			6,7,8,9,10,11:				START_BIT <= START_BIT + s1_Rs232_Rx;
			22,23,24,25,26,27:		Data_byte_r[0] <= Data_byte_r[0] + s1_Rs232_Rx;//tmp1_Rs232_Rx是用于判断串口下降沿动作的，有滞后，因此用s1_Rs232_Rx
			38,39,40,41,42,43:		Data_byte_r[1] <= Data_byte_r[1] + s1_Rs232_Rx;
			54,55,56,57,58,59:		Data_byte_r[2] <= Data_byte_r[2] + s1_Rs232_Rx;
			70,71,72,73,74,75:		Data_byte_r[3] <= Data_byte_r[3] + s1_Rs232_Rx;
			86,87,88,89,90,91:		Data_byte_r[4] <= Data_byte_r[4] + s1_Rs232_Rx;
			102,103,104,105,106,107:Data_byte_r[5] <= Data_byte_r[5] + s1_Rs232_Rx;
			118,119,120,121,122,123:Data_byte_r[6] <= Data_byte_r[6] + s1_Rs232_Rx;
			134,135,136,137,138,139:Data_byte_r[7] <= Data_byte_r[7] + s1_Rs232_Rx;
			150,151,152,153,154,155:STOP_BIT <= STOP_BIT + s1_Rs232_Rx;
			default:;
		endcase 
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		Data_byte <= 8'd0;
	else if(bps_cnt == 8'd161)begin
		Data_byte[0] <= Data_byte_r[0][2];	
		Data_byte[1] <= Data_byte_r[1][2];	
		Data_byte[2] <= Data_byte_r[2][2];	
		Data_byte[3] <= Data_byte_r[3][2];	
		Data_byte[4] <= Data_byte_r[4][2];	
		Data_byte[5] <= Data_byte_r[5][2];	
		Data_byte[6] <= Data_byte_r[6][2];	
		Data_byte[7] <= Data_byte_r[7][2];
	end	                         
		
		
endmodule 
