`timescale 1ns/1ns
`define clk_period 20

module TFT_CTRL_tb;

	reg PCLK;
	reg Rst_n;
	reg [15:0]Data_in; 
	
	wire  TFT_HS;	//行同步信号
	wire  TFT_VS;	//场同步信号
	wire TFT_BL;	//背光 （可以是1K PWM，现在直接接高就行）
	wire [15:0]TFT_RGB;	//送给TFT显示的数据RGB565
	wire TFT_PCLK;	//显示屏像素驱动时钟 33.3MHZ
	wire Data_En;		//当扫到显示区800*480坐标时，通知外界开始向TFT送待显示数据
	wire [8:0]X_Count;		//显示区域有效数据的行坐标X 800
	wire [8:0]Y_Count;		//显示区域有效数据的列坐标Y 480
	wire Data_Req;
	
	TFT_CTRL TFT_CTRL_inst( //注意驱动TFT的时钟是33.3M，不是50M。顶层要加一个PLL
		.PCLK(PCLK), //TFT像素时钟33.3M 五寸屏
		.Rst_n(Rst_n),
		.Data_in(Data_in), 
		
		.TFT_HS(TFT_HS),	//行同步信号
		.TFT_VS(TFT_VS),	//场同步信号
		.TFT_BL(TFT_BL),	//背光 （可以是1K PWM，现在直接接高就行）
		.TFT_RGB(TFT_RGB),	//送给TFT显示的数据RGB565
		.TFT_PCLK(TFT_PCLK),	//显示屏像素驱动时钟 33.3MHZ
		.Data_En(Data_En),		//传输显示区域有效数据(800*480)
		.X_Count(X_Count),		//显示区域有效数据的行坐标X
		.Y_Count(Y_Count),		//显示区域有效数据的列坐标Y
		.Data_Req(Data_Req)
	);
	
	initial PCLK = 1;
	always #(`clk_period/2) PCLK = ~PCLK;

	initial begin
		Rst_n = 0;
		
		#201;
		Rst_n = 1;
		#200;
		repeat(1056*525)begin
			//Data_in = Data_in + 1;
			#20;
		end
		#(`clk_period*80);
		Data_in = 0;
		repeat(525*286)begin
			//Data_in = Data_in + 1;
			#20;
		end
		
		#220;
		
		$stop;
		
	
	end
	
//	always@(posedge PCLK or negedge Rst_n)
//	if(!Rst_n)
//		Data_in = 0;
//	else if(Data_En) //当TFT需要外界送数据时才产生Data_in，所以TFT_RGB值在Data_En高电平时应从0开始到799
//		Data_in <= Data_in + 1;
	
	always@(posedge PCLK or negedge Rst_n)
	if(!Rst_n)
		Data_in = 0;
	else if(Data_En) //当TFT需要外界送数据时才产生Data_in，所以TFT_RGB值在Data_En高电平时应从0开始到799
		Data_in <= Data_in + 1;
	
endmodule 