`timescale 1ns/1ns
`define wrclk_period 40
`define rdclk_period 20

module dpram_tb; //双端口双时钟，位宽均为16位RGB 因为串口写的满，读得快

	reg	[15:0]  data;
	reg	  rd_aclr;
	reg	[15:0]  rdaddress;
	reg	  rdclock;
	reg	  rden;
	reg	[15:0]  wraddress;
	reg	  wrclock;
	reg	  wren;
	
	wire	[15:0]  q;




//经过仿真发现写地址和数据对应没有延迟，而读地址和读出数据q有延迟，读地址改变后读出数据延迟一拍改变，因此只能在后推一个时钟周期后级模块才能取用到正确数据。即读地址变换后延迟两拍取到的q才是有效的数据
dpram	dpram_inst (//因此当读使能无效后由于RAM读出数据变化滞后读地址一拍，所以在读使能无效后仍然会接着输出一个数据，保证数据个数完整性。而且读使能无效后输出q一直保持最后的65535值不变。
	.data ( data ),
	.rd_aclr ( rd_aclr ),
	.rdaddress ( rdaddress ),
	.rdclock ( rdclock ),
	.rden ( rden ),
	
	.wraddress ( wraddress ),
	.wrclock ( wrclock ),
	.wren ( wren ),
	.q ( q )
	);
	
	initial wrclock = 1;
	always #(`wrclk_period/2) wrclock = ~wrclock;
	
	initial rdclock = 1;
	always #(`rdclk_period/2) rdclock = ~rdclock;

	initial begin
		rd_aclr = 1;
		data = 0;
		rden = 0;
		wren = 0;
		rdaddress = 0;
		wraddress = 0;
		#201;
		rd_aclr = 0;
		#200;
		wren = 1;
		repeat(256*256)begin			
			#40;
			data = data + 1;
			wraddress = wraddress + 1;
		end
		wren = 0;
		repeat(6)begin			
			#40;
			data = data + 1;
			wraddress = wraddress + 1;
		end
		//wren = 0;
		#(`wrclk_period*80);
		rden = 1;
		repeat(256*256)begin			
			#20;
			rdaddress = rdaddress + 1;
		end
		rden = 0;
		repeat(6)begin		//多读几个数看看是否会出错	
			#20;
			rdaddress = rdaddress + 1;
		end
		rd_aclr = 1;
		
		#220;		
		$stop;
		
	
	end
	
	
	
	
endmodule 
